@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -25,13 +25,9 @@
 #include <config.h>
 #endif
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
-#include <camel/camel-provider.h>
-#include <camel/camel-url.h>
-#include <camel/camel-service.h>
-#include <camel/camel-folder.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libedataserverui/e-passwords.h>
 #include <libedataserver/e-account.h>
@@ -50,7 +46,6 @@
 #include <mail/mail-config.h>
 #include <mail/em-config.h>
 #include <mail/em-folder-tree.h>
-#include <misc/e-spinner.h>
 
 #include "exchange-mapi-account-listener.h"
 
@@ -60,7 +55,6 @@ gboolean  e_plugin_ui_init (GtkUIManager *ui_manager,
 			    EShellView *shell_view);
 
 GtkWidget *org_gnome_exchange_mapi_settings (EPlugin *epl, EConfigHookItemFactoryData *data);
-void mapi_settings_run_folder_size_dialog (gpointer data);
 
 enum {
 	COL_FOLDERSIZE_NAME = 0,
@@ -74,6 +68,8 @@ typedef struct
 	GtkWidget *spinner;
 	GtkWidget *spinner_label;
 	GtkBox *spinner_hbox;
+
+	gchar *profile;
 } FolderSizeDialogData;
 
 static gboolean
@@ -86,8 +82,13 @@ mapi_settings_get_folder_size (gpointer data)
 	GtkTreeIter iter;
 	GtkBox *content_area;
 	FolderSizeDialogData *dialog_data = (FolderSizeDialogData *)data;
+	GSList *folder_list;
+	ExchangeMapiConnection *conn;
 
-	GSList *folder_list = exchange_mapi_account_listener_peek_folder_list ();
+	folder_list = NULL;
+	conn = exchange_mapi_connection_find (dialog_data->profile);
+	if (conn && exchange_mapi_connection_connected (conn))
+		folder_list = exchange_mapi_connection_peek_folders_list (conn);
 
 	/* Hide progress bar. Set status*/
 	gtk_widget_destroy (dialog_data->spinner_label);
@@ -97,25 +98,25 @@ mapi_settings_get_folder_size (gpointer data)
 		/*Tree View */
 		view =  gtk_tree_view_new ();
 		renderer = gtk_cell_renderer_text_new ();
-		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),-1, 
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),-1,
 							     _("Folder"), renderer, "text", COL_FOLDERSIZE_NAME,
 							     NULL);
-		
+
 		renderer = gtk_cell_renderer_text_new ();
-		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),-1, 
+		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),-1,
 							     _("Size"), renderer, "text", COL_FOLDERSIZE_SIZE,
 							     NULL);
 		/* Model for TreeView */
 		store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 		gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
-		
+
 		/* Populate model with data */
 		while (folder_list) {
 			ExchangeMAPIFolder *folder = (ExchangeMAPIFolder *) folder_list->data;
 			gchar *folder_size = g_format_size_for_display (folder->size);
-			
+
 			gtk_list_store_append (store, &iter);
-			gtk_list_store_set (store, &iter, 
+			gtk_list_store_set (store, &iter,
 					    COL_FOLDERSIZE_NAME, folder->folder_name,
 					    COL_FOLDERSIZE_SIZE, folder_size,
 					    -1);
@@ -130,13 +131,16 @@ mapi_settings_get_folder_size (gpointer data)
 
 	/* Pack into content_area */
 	content_area = (GtkBox*) gtk_dialog_get_content_area (dialog_data->dialog);
-	gtk_box_pack_start (content_area, view, TRUE, TRUE, 6);	
+	gtk_box_pack_start (content_area, view, TRUE, TRUE, 6);
+
+	if (conn)
+		g_object_unref (conn);
 
 	return FALSE;
 }
 
-void
-mapi_settings_run_folder_size_dialog (gpointer data)
+static void
+mapi_settings_run_folder_size_dialog (const gchar *profile, gpointer data)
 {
 	GtkBox *content_area;
 	FolderSizeDialogData *dialog_data;
@@ -144,16 +148,16 @@ mapi_settings_run_folder_size_dialog (gpointer data)
 
 	dialog_data = g_new0 (FolderSizeDialogData, 1);
 
-	dialog_data->dialog = (GtkDialog *)gtk_dialog_new_with_buttons (_("Folder Size"), NULL, 
+	dialog_data->dialog = (GtkDialog *)gtk_dialog_new_with_buttons (_("Folder Size"), NULL,
 							   GTK_DIALOG_DESTROY_WITH_PARENT,
 							   GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
 							   NULL);
 
 	content_area = (GtkBox *)gtk_dialog_get_content_area (dialog_data->dialog);
 
-	/* dialog_data->spinner = g_object_new (E_TYPE_SPINNER, NULL); */
-	dialog_data->spinner = e_spinner_new_spinning_small_shown ();
-	dialog_data->spinner_label = gtk_label_new (_("Fetching folder list ..."));
+	dialog_data->spinner = gtk_spinner_new ();
+	gtk_spinner_start (GTK_SPINNER (dialog_data->spinner));
+	dialog_data->spinner_label = gtk_label_new (_("Fetching folder list…"));
 
 	dialog_data->spinner_hbox = (GtkBox *) gtk_hbox_new (TRUE, 6);
 
@@ -164,6 +168,8 @@ mapi_settings_run_folder_size_dialog (gpointer data)
 	gtk_box_pack_start (content_area, GTK_WIDGET (dialog_data->spinner_hbox), TRUE, TRUE, 6);
 	gtk_widget_show_all (GTK_WIDGET (dialog_data->dialog));
 
+	dialog_data->profile = g_strdup (profile);
+
 	/* Fetch folder list and size information in a thread */
 	folder_list_thread = g_thread_create ((GThreadFunc)mapi_settings_get_folder_size, dialog_data, TRUE, NULL);
 
@@ -171,13 +177,26 @@ mapi_settings_run_folder_size_dialog (gpointer data)
 	gtk_dialog_run (dialog_data->dialog);
 
 	gtk_widget_destroy (GTK_WIDGET (dialog_data->dialog));
+
+	g_free (dialog_data->profile);
 	g_free (dialog_data);
 }
 
 static void
-folder_size_clicked (GtkButton *button, gpointer data)
+folder_size_clicked (GtkButton *button, EAccount *account)
 {
-	mapi_settings_run_folder_size_dialog (NULL);
+	CamelURL *url;
+
+	g_return_if_fail (account != NULL);
+	g_return_if_fail (E_IS_ACCOUNT (account));
+
+	
+	url = camel_url_new (e_account_get_string (account,  E_ACCOUNT_SOURCE_URL), NULL);
+	g_return_if_fail (url != NULL);
+
+	mapi_settings_run_folder_size_dialog (camel_url_get_param (url, "profile"), NULL);
+
+	camel_url_free (url);
 }
 
 static void
@@ -187,16 +206,50 @@ action_folder_size_cb (GtkAction *action,
 	EShellSidebar *shell_sidebar;
 	EMFolderTree *folder_tree;
 	const gchar *folder_uri;
+	GtkTreeSelection *selection;
+	gchar *profile = NULL;
 
 	/* Get hold of Folder Tree */
 	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
 	g_object_get (shell_sidebar, "folder-tree", &folder_tree, NULL);
 	folder_uri = em_folder_tree_get_selected_uri (folder_tree);
+	selection = em_folder_tree_model_get_selection (EM_FOLDER_TREE_MODEL (gtk_tree_view_get_model (GTK_TREE_VIEW (folder_tree))));
+	if (selection) {
+		GtkTreeIter iter;
+		GtkTreeModel *model = NULL;
+
+		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+			gboolean is_folder = FALSE;
+			CamelStore *store = NULL;
+
+			gtk_tree_model_get (model, &iter,
+				COL_BOOL_IS_FOLDER, &is_folder,
+				COL_POINTER_CAMEL_STORE, &store,
+				-1);
+
+			if (is_folder && !store) {
+				CamelFolder *folder = em_folder_tree_get_selected_folder (folder_tree);
+
+				if (folder)
+					store = camel_folder_get_parent_store (folder);
+			}
+
+			if (store && CAMEL_IS_SERVICE (store)) {
+				CamelService *service = CAMEL_SERVICE (store);
+
+				if (service->url)
+					profile = g_strdup (camel_url_get_param (service->url, "profile"));
+			}
+		}
+	}
+
 	g_object_unref (folder_tree);
 	g_return_if_fail (folder_uri != NULL);
 
 	if (g_str_has_prefix (folder_uri, "mapi://"))
-		mapi_settings_run_folder_size_dialog (NULL);
+		mapi_settings_run_folder_size_dialog (profile, NULL);
+
+	g_free (profile);
 }
 
 static void
@@ -211,6 +264,8 @@ folder_size_actions_update_cb (EShellView *shell_view, GtkActionEntry *entries)
 	EShellSidebar *shell_sidebar;
 	EMFolderTree *folder_tree;
 	const gchar *folder_uri = NULL;
+	CamelURL *url = NULL;
+	gboolean show_menu_entry = FALSE;
 
 	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
 	g_object_get (shell_sidebar, "folder-tree", &folder_tree, NULL);
@@ -224,16 +279,20 @@ folder_size_actions_update_cb (EShellView *shell_view, GtkActionEntry *entries)
 
 	ui_manager = e_shell_window_get_ui_manager (shell_window);
 	action_group = e_lookup_action_group (ui_manager, "mail");
-		
-	folder_size_action = gtk_action_group_get_action (action_group, 
+
+	folder_size_action = gtk_action_group_get_action (action_group,
 							  "mail-mapi-folder-size");
 
 	/* Show / Hide action entry */
-	if (g_str_has_prefix (folder_uri, "mapi://"))
-		gtk_action_set_visible (folder_size_action , TRUE);
-	else 
-		gtk_action_set_visible (folder_size_action , FALSE);
+	if (g_str_has_prefix (folder_uri, "mapi://")) {
+		show_menu_entry = TRUE;
+		url = camel_url_new (folder_uri, NULL);
+		if (url && *url->path && strlen (url->path) > 1)
+			show_menu_entry = FALSE;
+		camel_url_free (url);
+	}
 
+	gtk_action_set_visible (folder_size_action, show_menu_entry);
 }
 
 /* used only in Account Editor */
@@ -274,7 +333,7 @@ org_gnome_exchange_mapi_settings (EPlugin *epl, EConfigHookItemFactoryData *data
 	gtk_container_set_border_width (GTK_CONTAINER (vbox_misc), 6);
 	gtk_container_add (GTK_CONTAINER (frm_misc), GTK_WIDGET (vbox_misc));
 
-	tbl_misc = (GtkTable*) g_object_new (GTK_TYPE_TABLE, "n-rows", 1, "n-columns", 1, 
+	tbl_misc = (GtkTable*) g_object_new (GTK_TYPE_TABLE, "n-rows", 1, "n-columns", 1,
 					     "homogeneous", FALSE, "row-spacing", 6,
 					     "column-spacing", 6, NULL);
 
@@ -282,8 +341,8 @@ org_gnome_exchange_mapi_settings (EPlugin *epl, EConfigHookItemFactoryData *data
 	lbl_fsize = (GtkLabel*) g_object_new (GTK_TYPE_LABEL, "label",
 					      _("View the size of all Exchange folders"), NULL);
 	gtk_misc_set_alignment (GTK_MISC (lbl_fsize), 0, 0.5);
-	btn_fsize = (GtkButton*) g_object_new (GTK_TYPE_BUTTON, "label", _("Folders Size"), NULL);
-	g_signal_connect (btn_fsize, "clicked", G_CALLBACK (folder_size_clicked), NULL);
+	btn_fsize = (GtkButton*) g_object_new (GTK_TYPE_BUTTON, "label", _("Folder Size"), NULL);
+	g_signal_connect (btn_fsize, "clicked", G_CALLBACK (folder_size_clicked), target_account->account);
 	gtk_table_attach_defaults (tbl_misc, GTK_WIDGET (lbl_fsize), 0, 1, 0, 1);
 	gtk_table_attach (tbl_misc, GTK_WIDGET (btn_fsize), 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 
