@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -28,13 +28,9 @@
 
 #include <string.h>
 #include <unistd.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <gtk/gtk.h>
-#include <camel/camel-provider.h>
-#include <camel/camel-url.h>
-#include <camel/camel-service.h>
-#include <camel/camel-folder.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libedataserverui/e-passwords.h>
 #include <libedataserver/e-account.h>
@@ -50,14 +46,15 @@
 
 #define d(x) x
 
-int e_plugin_lib_enable (EPlugin *ep, int enable);
+gint e_plugin_lib_enable (EPlugin *ep, gint enable);
 
 /* Account Setup */
 GtkWidget *org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData *data);
 gboolean org_gnome_exchange_mapi_check_options(EPlugin *epl, EConfigHookPageCheckData *data);
 
 /* New Addressbook/CAL */
-GtkWidget *exchange_mapi_create (EPlugin *epl, EConfigHookItemFactoryData *data);
+GtkWidget *exchange_mapi_create_addressbook (EPlugin *epl, EConfigHookItemFactoryData *data);
+GtkWidget *exchange_mapi_create_calendar (EPlugin *epl, EConfigHookItemFactoryData *data);
 
 /* New Addressbook */
 gboolean exchange_mapi_book_check (EPlugin *epl, EConfigHookPageCheckData *data);
@@ -67,23 +64,22 @@ void exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target);
 gboolean exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data);
 void exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target);
 
-
 static ExchangeMAPIAccountListener *config_listener = NULL;
 
-static void 
+static void
 free_mapi_listener ( void )
 {
 	g_object_unref (config_listener);
 }
 
-int
-e_plugin_lib_enable (EPlugin *ep, int enable)
+gint
+e_plugin_lib_enable (EPlugin *ep, gint enable)
 {
 	g_debug ("Loading Exchange MAPI Plugin \n");
 
 	if (!config_listener) {
 		config_listener = exchange_mapi_account_listener_new ();
-	 	g_atexit ( free_mapi_listener );
+		g_atexit ( free_mapi_listener );
 	}
 
 	return 0;
@@ -92,7 +88,7 @@ e_plugin_lib_enable (EPlugin *ep, int enable)
 ExchangeMAPIAccountListener *
 exchange_mapi_accounts_peek_config_listener ()
 {
-	return config_listener; 
+	return config_listener;
 }
 
 enum {
@@ -102,7 +98,7 @@ enum {
   COLS_MAX
 };
 
-/* Callback for ProcessNetworkProfile. If we have more than one username, 
+/* Callback for ProcessNetworkProfile. If we have more than one username,
  we need to let the user select. */
 static uint32_t
 create_profile_callback (struct SRowSet *rowset, gpointer data)
@@ -120,7 +116,9 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 
 	/* If we can find the exact username, then find & return its index. */
 	for (i = 0; i < rowset->cRows; i++) {
-		lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT);
+		lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT_UNICODE);
+		if (!lpProp_account)
+			lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT);
 
 		if (lpProp_account && lpProp_account->value.lpszA &&
 		    !g_strcmp0 (username, lpProp_account->value.lpszA))
@@ -147,7 +145,7 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
-						     -1, _("User name"), renderer, 
+						     -1, _("Username"), renderer,
 						     "text", COL_MAPI_ACCOUNT, NULL);
 
 	/* Model for TreeView */
@@ -155,8 +153,12 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
 
 	for (i = 0; i < rowset->cRows; i++) {
-		lpProp_fullname = get_SPropValue_SRow(&(rowset->aRow[i]), PR_DISPLAY_NAME);
-		lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT);
+		lpProp_fullname = get_SPropValue_SRow(&(rowset->aRow[i]), PR_DISPLAY_NAME_UNICODE);
+		if (!lpProp_fullname)
+			lpProp_fullname = get_SPropValue_SRow(&(rowset->aRow[i]), PR_DISPLAY_NAME);
+		lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT_UNICODE);
+		if (!lpProp_account)
+			lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT);
 
 		if (lpProp_fullname && lpProp_fullname->value.lpszA &&
 		    lpProp_account && lpProp_account->value.lpszA) {
@@ -179,7 +181,7 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	       /* Get the index from the selected value */
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 		gtk_tree_selection_get_selected (selection, NULL, &iter);
-		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX, 
+		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX,
 				    &index, -1);
 	} else /* If we return a value > available, we are canceling the login.*/
 	       index = rowset->cRows + 1;
@@ -194,13 +196,13 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 {
 	EMConfigTargetAccount *target_account = (EMConfigTargetAccount *)(config->target);
 	CamelURL *url = NULL;
- 	gchar *key = NULL, *password = NULL;
-	const gchar *domain_name = NULL; 
+	gchar *key = NULL, *password = NULL;
+	const gchar *domain_name = NULL;
 
 	url = camel_url_new (e_account_get_string (target_account->account, E_ACCOUNT_SOURCE_URL), NULL);
 	domain_name = camel_url_get_param (url, "domain");
 
-	/* Silently remove domain part from a user name when user enters it as such.
+	/* Silently remove domain part from a username when user enters it as such.
 	   This change will be visible in the UI on new edit open. */
 	if (url->user && strchr (url->user, '\\')) {
 		gchar *tmp, *at;
@@ -232,17 +234,34 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 	/*Can there be a account without password ?*/
 	if (password && *password && domain_name && *domain_name && *url->user && *url->host) {
-		char *error_msg = NULL;
+		gchar *error_msg = NULL;
 		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name,
-								url->host, &error_msg, 
+								url->host, &error_msg,
 								(mapi_profile_callback_t) create_profile_callback,
 								url->user);
 		if (status) {
-			/* Things are successful */
-			gchar *profname = NULL, *uri = NULL; 
+			/* profile was created, try to connect to the server */
+			ExchangeMapiConnection *conn;
+			gchar *profname;
 
-			profname = exchange_mapi_util_profile_name (url->user, domain_name);
-			camel_url_set_param(url, "profile", profname);
+			status = FALSE;
+			profname = exchange_mapi_util_profile_name (url->user, domain_name, url->host, FALSE);
+
+			conn = exchange_mapi_connection_new (profname, password);
+			if (conn) {
+				status = exchange_mapi_connection_connected (conn);
+				g_object_unref (conn);
+			}
+
+			g_free (profname);
+		}
+
+		if (status) {
+			/* Things are successful */
+			gchar *profname = NULL, *uri = NULL;
+
+			profname = exchange_mapi_util_profile_name (url->user, domain_name, url->host, FALSE);
+			camel_url_set_param (url, "profile", profname);
 			g_free (profname);
 
 			uri = camel_url_to_string(url, 0);
@@ -252,7 +271,7 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 			e_notice (NULL, GTK_MESSAGE_INFO, "%s", _("Authentication finished successfully."));
 		} else {
-			char *e;
+			gchar *e;
 
 			e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
 
@@ -279,8 +298,8 @@ domain_entry_changed(GtkWidget *entry, EConfig *config)
 {
 	EMConfigTargetAccount *target = (EMConfigTargetAccount *)(config->target);
 	CamelURL *url = NULL;
-	const char *domain = NULL;
-	char *url_string = NULL;
+	const gchar *domain = NULL;
+	gchar *url_string = NULL;
 
 	url = camel_url_new (e_account_get_string(target->account, E_ACCOUNT_SOURCE_URL), NULL);
 	domain = gtk_entry_get_text (GTK_ENTRY(entry));
@@ -304,20 +323,21 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 	EMConfigTargetAccount *target_account;
 	CamelURL *url;
 	GtkWidget *hbox = NULL;
+	gint row;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
 	url = camel_url_new(e_account_get_string(target_account->account, E_ACCOUNT_SOURCE_URL), NULL);
 
 	/* is NULL on New Account creation */
 	if (url == NULL)
-		return NULL; 
+		return NULL;
 
 	if (!g_ascii_strcasecmp (url->protocol, "mapi")) {
 		GtkWidget *label;
 		GtkWidget *domain_name;
 		GtkWidget *auth_button;
-		const char *domain_value = camel_url_get_param (url, "domain");
-		int row = ((GtkTable *)data->parent)->nrows;
+		const gchar *domain_value = camel_url_get_param (url, "domain");
+		g_object_get (data->parent, "n-rows", &row, NULL);
 
 		/* Domain name & Authenticate Button */
 		hbox = gtk_hbox_new (FALSE, 6);
@@ -337,7 +357,7 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 
 		gtk_table_attach (GTK_TABLE (data->parent), label, 0, 1, row, row+1, 0, 0, 0, 0);
 		gtk_widget_show_all (GTK_WIDGET (hbox));
-		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hbox), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0); 
+		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hbox), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 	}
 
 	camel_url_free (url);
@@ -351,19 +371,19 @@ org_gnome_exchange_mapi_check_options(EPlugin *epl, EConfigHookPageCheckData *da
 	gboolean status = TRUE;
 
 	if (data->pageid != NULL && g_ascii_strcasecmp (data->pageid, "10.receive") == 0) {
-		CamelURL *url = camel_url_new (e_account_get_string(target->account,  
+		CamelURL *url = camel_url_new (e_account_get_string(target->account,
 								    E_ACCOUNT_SOURCE_URL), NULL);
 
 		if (url && url->protocol && g_ascii_strcasecmp (url->protocol, "mapi") == 0) {
 			const gchar *prof = NULL;
 
 			/* We assume that if the profile is set, then the setting is valid. */
- 			prof = camel_url_get_param (url, "profile");
+			prof = camel_url_get_param (url, "profile");
 
 			/*Profile not set. Do not proceed with account creation.*/
 			if (!(prof && *prof))
-			        status = FALSE;
-		} 
+				status = FALSE;
+		}
 
 		if (url)
 			camel_url_free(url);
@@ -373,38 +393,37 @@ org_gnome_exchange_mapi_check_options(EPlugin *epl, EConfigHookPageCheckData *da
 }
 
 enum {
-	CONTACTSNAME_COL,
-	CONTACTSFID_COL,
-	CONTACTSFOLDER_COL,
+	NAME_COL,
+	FID_COL,
+	FOLDER_COL,
 	NUM_COLS
 };
 
-
 static gboolean
-check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter *iter)
+check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter iter)
 {
 	GtkTreeModel *ts_model;
 	mapi_id_t fid;
 	gboolean status = FALSE;
 
 	ts_model = GTK_TREE_MODEL (ts);
-	
-	gtk_tree_model_get (ts_model, iter, 1, &fid, -1);
+
+	gtk_tree_model_get (ts_model, &iter, 1, &fid, -1);
 	if (fid && folder->parent_folder_id == fid) {
 		/* Do something */
 		GtkTreeIter node;
-		gtk_tree_store_append (ts, &node, iter);		
-		gtk_tree_store_set (ts, &node, 0, folder->folder_name, 1, folder->folder_id, 2, folder,-1);		
+		gtk_tree_store_append (ts, &node, &iter);
+		gtk_tree_store_set (ts, &node, NAME_COL, folder->folder_name, FID_COL, folder->folder_id, FOLDER_COL, folder,-1);
 		return TRUE;
 	}
 
-	if (gtk_tree_model_iter_has_child (ts_model, iter)) {
+	if (gtk_tree_model_iter_has_child (ts_model, &iter)) {
 		GtkTreeIter child;
-		gtk_tree_model_iter_children (ts_model, &child, iter);
-		status = check_node (ts, folder, &child);
+		gtk_tree_model_iter_children (ts_model, &child, &iter);
+		status = check_node (ts, folder, child);
 	}
 
-	while (gtk_tree_model_iter_next (ts_model, iter) && !status) {
+	while (gtk_tree_model_iter_next (ts_model, &iter) && !status) {
 		status = check_node (ts, folder, iter);
 	}
 
@@ -418,30 +437,124 @@ add_to_store (GtkTreeStore *ts, ExchangeMAPIFolder *folder)
 	GtkTreeIter iter;
 
 	ts_model = GTK_TREE_MODEL (ts);
-	
+
 	gtk_tree_model_get_iter_first (ts_model, &iter);
-	if (!check_node (ts, folder, &iter)) {
+	if (!check_node (ts, folder, iter)) {
 		GtkTreeIter node;
-		gtk_tree_store_append (ts, &node, &iter);		
-		gtk_tree_store_set (ts, &node, 0, folder->folder_name, 1, folder->folder_id, -1);
-		
+		gtk_tree_store_append (ts, &node, &iter);
+		gtk_tree_store_set (ts, &node, NAME_COL, folder->folder_name, FID_COL, folder->folder_id, FOLDER_COL, folder, -1);
 	}
 }
 
 static void
-add_folders (GSList *folders, GtkTreeStore *ts)
+traverse_tree (GtkTreeModel *model, GtkTreeIter iter, ExchangeMAPIFolderType folder_type, gboolean *pany_sub_used)
+{
+	gboolean any_sub_used = FALSE;
+	gboolean has_next = TRUE;
+
+	do {
+		gboolean sub_used = FALSE;
+		GtkTreeIter next = iter;
+		ExchangeMAPIFolder *folder = NULL;
+
+		has_next = gtk_tree_model_iter_next (model, &next);
+
+		if (gtk_tree_model_iter_has_child (model, &iter)) {
+			GtkTreeIter child;
+
+			gtk_tree_model_iter_children (model, &child, &iter);
+			traverse_tree (model, child, folder_type, &sub_used);
+		}
+
+		gtk_tree_model_get (model, &iter, FOLDER_COL, &folder, -1);
+		if (folder && (exchange_mapi_folder_get_type (folder) == folder_type || (folder_type == MAPI_FOLDER_TYPE_MEMO && exchange_mapi_folder_get_type (folder) == MAPI_FOLDER_TYPE_JOURNAL))) {
+			sub_used = TRUE;
+		}
+
+		if (sub_used)
+			any_sub_used = TRUE;
+		else if (pany_sub_used && folder)
+			gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+
+		iter = next;
+	} while (has_next);
+
+	if (pany_sub_used && any_sub_used)
+		*pany_sub_used = TRUE;
+}
+
+static void
+add_folders (GSList *folders, GtkTreeStore *ts, ExchangeMAPIFolderType folder_type)
 {
 	GSList *tmp = folders;
 	GtkTreeIter iter;
-	char *node = _("Personal Folders");
-	
+	gchar *node = _("Personal Folders");
+
+	/* add all... */
 	gtk_tree_store_append (ts, &iter, NULL);
-	gtk_tree_store_set (ts, &iter, 0, node, -1);
+	gtk_tree_store_set (ts, &iter, NAME_COL, node, -1);
 	while (tmp) {
 		ExchangeMAPIFolder *folder = tmp->data;
 		add_to_store (ts, folder);
 		tmp = tmp->next;
 	}
+
+	/* ... then remove those which don't belong to folder_type */
+	if (gtk_tree_model_get_iter_first ((GtkTreeModel *)ts, &iter)) {
+		traverse_tree ((GtkTreeModel *)ts, iter, folder_type, NULL);
+	}
+}
+
+static void
+select_folder (GtkTreeModel *model, mapi_id_t fid, GtkWidget *tree_view)
+{
+	GtkTreeIter iter, next;
+	gboolean found = FALSE, can = TRUE;
+
+	g_return_if_fail (model != NULL);
+	g_return_if_fail (tree_view != NULL);
+
+	if (!gtk_tree_model_get_iter_first (model, &iter))
+		return;
+
+	while (!found && can) {
+		ExchangeMAPIFolder *folder = NULL;
+
+		gtk_tree_model_get (model, &iter, FOLDER_COL, &folder, -1);
+
+		if (folder && exchange_mapi_folder_get_fid (folder) == fid) {
+			gtk_tree_selection_select_iter (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)), &iter);
+			found = TRUE;
+			break;
+		}
+
+		can = FALSE;
+		if (gtk_tree_model_iter_children (model, &next, &iter)) {
+			iter = next;
+			can = TRUE;
+		}
+
+		next = iter;
+		if (!can && gtk_tree_model_iter_next (model, &next)) {
+			iter = next;
+			can = TRUE;
+		}
+
+		if (!can && gtk_tree_model_iter_parent (model, &next, &iter)) {
+			while (!can) {
+				iter = next;
+
+				if (gtk_tree_model_iter_next (model, &iter)) {
+					can = TRUE;
+					break;
+				}
+
+				iter = next;
+				if (!gtk_tree_model_iter_parent (model, &next, &iter))
+					break;
+			}
+		}
+	}
 }
 
 static void
@@ -452,68 +565,81 @@ exchange_mapi_cursor_change (GtkTreeView *treeview, ESource *source)
 	GtkTreeIter       iter;
 	mapi_id_t pfid;
 	gchar *sfid=NULL;
-	
+
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
 	gtk_tree_selection_get_selected(selection, &model, &iter);
 
-	gtk_tree_model_get (model, &iter, CONTACTSFID_COL, &pfid, -1);
+	gtk_tree_model_get (model, &iter, FID_COL, &pfid, -1);
 	sfid = exchange_mapi_util_mapi_id_to_string (pfid);
-	e_source_set_property (source, "parent-fid", sfid); 
+	e_source_set_property (source, "parent-fid", sfid);
 	g_free (sfid);
 }
 
-GtkWidget *
-exchange_mapi_create (EPlugin *epl, EConfigHookItemFactoryData *data)
+static GtkWidget *
+exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType folder_type)
 {
 	GtkWidget *vbox, *label, *scroll, *tv;
-	EABConfigTargetSource *t = (EABConfigTargetSource *) data->target;
-	ESource *source = t->source;
-	char *uri_text;
+	gchar *uri_text;
+	gint row;
 	GtkCellRenderer *rcell;
 	GtkTreeStore *ts;
 	GtkTreeViewColumn *tvc;
-	const char *acc;
+	const gchar *acc;
 	GSList *folders;
+	ExchangeMapiConnection *conn;
+	mapi_id_t fid = 0;
 
 	uri_text = e_source_get_uri (source);
 	if (uri_text && g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH)) {
 		return NULL;
 	}
 
-	folders = exchange_mapi_account_listener_peek_folder_list ();
+	folders = NULL;
+	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	if (conn && exchange_mapi_connection_connected (conn))
+		folders = exchange_mapi_connection_peek_folders_list (conn);
+
 	acc = e_source_group_peek_name (e_source_peek_group (source));
 	ts = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_INT64, G_TYPE_POINTER);
 
-	add_folders (folders, ts);
-	
+	add_folders (folders, ts, folder_type);
+
+	if (conn)
+		g_object_unref (conn);
+
 	vbox = gtk_vbox_new (FALSE, 6);
 
-	if (!strcmp (data->config->id, "org.gnome.evolution.calendar.calendarProperties")) {
-		int row = ((GtkTable*) data->parent)->nrows;
-		gtk_table_attach (GTK_TABLE (data->parent), vbox, 0, 2, row+1, row+2, GTK_FILL|GTK_EXPAND, 0, 0, 0);
-	} else if (!strcmp (data->config->id, "com.novell.evolution.addressbook.config.accountEditor")) {
-		gtk_container_add (GTK_CONTAINER (data->parent), vbox);
+	if (folder_type == MAPI_FOLDER_TYPE_CONTACT) {
+		gtk_container_add (GTK_CONTAINER (parent), vbox);
+	} else {
+		g_object_get (parent, "n-rows", &row, NULL);
+		gtk_table_attach (GTK_TABLE (parent), vbox, 0, 2, row+1, row+2, GTK_FILL|GTK_EXPAND, 0, 0, 0);
 	}
 
 	label = gtk_label_new_with_mnemonic (_("_Location:"));
 	gtk_widget_show (label);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-	
+
 	rcell = gtk_cell_renderer_text_new ();
-	tvc = gtk_tree_view_column_new_with_attributes (acc, rcell, "text", CONTACTSNAME_COL, NULL);
+	tvc = gtk_tree_view_column_new_with_attributes (acc, rcell, "text", NAME_COL, NULL);
 	tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (ts));
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tv), tvc);
 	g_object_set (tv,"expander-column", tvc, "headers-visible", TRUE, NULL);
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (tv));
-	
+
+	if (e_source_get_property (source, "folder-id")) {
+		exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "folder-id"), &fid);
+		select_folder (GTK_TREE_MODEL (ts), fid, tv);
+	}
+
 	scroll = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
 	g_object_set (scroll, "height-request", 150, NULL);
 	gtk_container_add (GTK_CONTAINER (scroll), tv);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), tv);
-	g_signal_connect (G_OBJECT (tv), "cursor-changed", G_CALLBACK (exchange_mapi_cursor_change), t->source);
+	g_signal_connect (G_OBJECT (tv), "cursor-changed", G_CALLBACK (exchange_mapi_cursor_change), source);
 	gtk_widget_show_all (scroll);
 
 	gtk_box_pack_start (GTK_BOX (vbox), scroll, FALSE, FALSE, 0);
@@ -522,12 +648,43 @@ exchange_mapi_create (EPlugin *epl, EConfigHookItemFactoryData *data)
 	return vbox;
 }
 
+GtkWidget *
+exchange_mapi_create_addressbook (EPlugin *epl, EConfigHookItemFactoryData *data)
+{
+	EABConfigTargetSource *t = (EABConfigTargetSource *) data->target;
+
+	return exchange_mapi_create (data->parent, t->source, MAPI_FOLDER_TYPE_CONTACT);
+}
+
+GtkWidget *
+exchange_mapi_create_calendar (EPlugin *epl, EConfigHookItemFactoryData *data)
+{
+	ECalConfigTargetSource *t = (ECalConfigTargetSource *) data->target;
+	ExchangeMAPIFolderType folder_type;
+
+	switch (t->source_type) {
+	case E_CAL_SOURCE_TYPE_EVENT:
+		folder_type = MAPI_FOLDER_TYPE_APPOINTMENT;
+		break;
+	case E_CAL_SOURCE_TYPE_TODO:
+		folder_type = MAPI_FOLDER_TYPE_TASK;
+		break;
+	case E_CAL_SOURCE_TYPE_JOURNAL:
+		folder_type = MAPI_FOLDER_TYPE_MEMO;
+		break;
+	default:
+		g_return_val_if_reached (NULL);
+	}
+
+	return exchange_mapi_create (data->parent, t->source, folder_type);
+}
+
 gboolean
 exchange_mapi_book_check (EPlugin *epl, EConfigHookPageCheckData *data)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) data->target;
 	ESource *source = t->source;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text)
 		return TRUE;
@@ -550,18 +707,18 @@ exchange_mapi_book_check (EPlugin *epl, EConfigHookPageCheckData *data)
 	return TRUE;
 }
 
-void 
+void
 exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) target;
 	ESource *source = t->source;
-	char *uri_text;
+	gchar *uri_text;
 	ESourceGroup *grp;
-	
+
 	uri_text = e_source_get_uri (source);
 	if (uri_text && g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
-	
+
 	//FIXME: Offline handling
 	grp = e_source_peek_group (source);
 	e_source_set_property (source, "auth", "plain/password");
@@ -578,24 +735,23 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 	return;
 }
 
-
 /* New calendar/task list/memo list */
 gboolean
 exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data)
 {
 	ECalConfigTargetSource *t = (ECalConfigTargetSource *)(data->target);
 	ESource *source = t->source;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text)
-		return TRUE; 
+		return TRUE;
 
 	/* FIXME: Offline handling */
 
 	/* not a MAPI account */
 	if (g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH)) {
-		g_free (uri_text); 
-		return TRUE; 
+		g_free (uri_text);
+		return TRUE;
 	}
 
 	g_free (uri_text);
@@ -609,32 +765,33 @@ exchange_mapi_cal_check (EPlugin *epl, EConfigHookPageCheckData *data)
 	return TRUE;
 }
 
-void 
+void
 exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 {
+	ExchangeMapiConnection *conn;
 	ECalConfigTargetSource *t = (ECalConfigTargetSource *) target;
 	ESourceGroup *group;
 	ESource *source = t->source;
 	gchar *tmp, *sfid;
 	mapi_id_t fid, pfid;
 	uint32_t type;
-	char *uri_text = e_source_get_uri (source);
+	gchar *uri_text = e_source_get_uri (source);
 
 	if (!uri_text || g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH))
 		return;
 	g_free (uri_text);
 
 	switch (t->source_type) {
-		case E_CAL_SOURCE_TYPE_EVENT: 
-			type = olFolderCalendar; 
+		case E_CAL_SOURCE_TYPE_EVENT:
+			type = olFolderCalendar;
 			break;
-		case E_CAL_SOURCE_TYPE_TODO: 
-			type = olFolderTasks; 
+		case E_CAL_SOURCE_TYPE_TODO:
+			type = olFolderTasks;
 			break;
-		case E_CAL_SOURCE_TYPE_JOURNAL: 
-			type = olFolderNotes; 
+		case E_CAL_SOURCE_TYPE_JOURNAL:
+			type = olFolderNotes;
 			break;
-		default: 
+		default:
 			g_warning ("%s: %s: Unknown ExchangeMAPIFolderType\n", G_STRLOC, G_STRFUNC);
 			return;
 	}
@@ -643,7 +800,12 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 
 	exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "parent-fid"), &pfid);
 
-	fid = exchange_mapi_create_folder (type, pfid, e_source_peek_name (source));
+	/* the profile should be already connected */
+	conn = exchange_mapi_connection_find (e_source_get_property (source, "profile"));
+	g_return_if_fail (conn != NULL);
+
+	fid = exchange_mapi_connection_create_folder (conn, type, pfid, e_source_peek_name (source));
+	g_object_unref (conn);
 
 	sfid = exchange_mapi_util_mapi_id_to_string (fid);
 	tmp = g_strconcat (";", sfid, NULL);
@@ -660,7 +822,7 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	tmp = e_source_group_get_property (group, "username");
 	e_source_set_property (source, "username", tmp);
 	g_free (tmp);
-	
+
 	tmp = e_source_group_get_property (group, "host");
 	e_source_set_property (source, "host", tmp);
 	g_free (tmp);
