@@ -1,5 +1,5 @@
 #ifdef HAVE_CONFIG_H
-#include <config.h>  
+#include <config.h>
 #endif
 
 #include <stdlib.h>
@@ -7,7 +7,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gstdio.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <sys/time.h>
 
@@ -20,107 +20,45 @@
 #include <libedata-book/e-data-book-view.h>
 #include <libedata-book/e-book-backend-cache.h>
 #include <libedata-book/e-book-backend-summary.h>
+
 #include "e-book-backend-mapi-gal.h"
+#include "e-book-backend-mapi-utils.h"
 
 G_DEFINE_TYPE (EBookBackendMAPIGAL, e_book_backend_mapi_gal, E_TYPE_BOOK_BACKEND)
 
 static gboolean enable_debug = TRUE;
-static GList *supported_fields;
-
-#define ELEMENT_TYPE_SIMPLE 0x01
-#define ELEMENT_TYPE_COMPLEX 0x02
-
-static const struct field_element_mapping {
-		EContactField field_id;
-		int element_type;
-	        int mapi_id;
-	        int contact_type;
-//		char *element_name;
-//		void (*populate_contact_func)(EContact *contact,    gpointer data);
-//		void (*set_value_in_gw_item) (EGwItem *item, gpointer data);
-//		void (*set_changes) (EGwItem *new_item, EGwItem *old_item);
-
-	} mappings [] = { 
-
-	{ E_CONTACT_UID, PT_UNICODE, 0, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_REV, PT_SYSTIME, PR_LAST_MODIFICATION_TIME, ELEMENT_TYPE_SIMPLE},
-		
-	{ E_CONTACT_FILE_AS, PT_UNICODE, PR_EMS_AB_MANAGER_T_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_FULL_NAME, PT_UNICODE, PR_DISPLAY_NAME_UNICODE, ELEMENT_TYPE_SIMPLE },
-	{ E_CONTACT_GIVEN_NAME, PT_UNICODE, PR_GIVEN_NAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_FAMILY_NAME, PT_UNICODE, PR_SURNAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_NICKNAME, PT_UNICODE, PR_NICKNAME_UNICODE, ELEMENT_TYPE_SIMPLE },
-
-	{ E_CONTACT_EMAIL_1, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x8084), ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_EMAIL_2, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x8094), ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_EMAIL_3, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x80a4), ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_IM_AIM,  PT_UNICODE, PROP_TAG(PT_UNICODE, 0x8062), ELEMENT_TYPE_COMPLEX},
-
-	{ E_CONTACT_PHONE_BUSINESS, PT_UNICODE, PR_OFFICE_TELEPHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_HOME, PT_UNICODE, PR_HOME_TELEPHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_MOBILE, PT_UNICODE, PR_MOBILE_TELEPHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_HOME_FAX, PT_UNICODE, PR_HOME_FAX_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_BUSINESS_FAX, PT_UNICODE, PR_BUSINESS_FAX_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_PAGER, PT_UNICODE, PR_PAGER_TELEPHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_ASSISTANT, PT_UNICODE, PR_ASSISTANT_TELEPHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_PHONE_COMPANY, PT_UNICODE, PR_COMPANY_MAIN_PHONE_NUMBER_UNICODE, ELEMENT_TYPE_SIMPLE},
-
-	{ E_CONTACT_HOMEPAGE_URL, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x802b), ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_FREEBUSY_URL, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x80d8), ELEMENT_TYPE_SIMPLE},
-
-	{ E_CONTACT_ROLE, PT_UNICODE, PR_PROFESSION_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_TITLE, PT_UNICODE, PR_TITLE_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_ORG, PT_UNICODE, PR_COMPANY_NAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_ORG_UNIT, PT_UNICODE, PR_DEPARTMENT_NAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_MANAGER, PT_UNICODE, PR_MANAGER_NAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_ASSISTANT, PT_UNICODE, PR_ASSISTANT_UNICODE, ELEMENT_TYPE_SIMPLE},
-
-	{ E_CONTACT_OFFICE, PT_UNICODE, PR_OFFICE_LOCATION_UNICODE, ELEMENT_TYPE_SIMPLE},
-	{ E_CONTACT_SPOUSE, PT_UNICODE, PR_SPOUSE_NAME_UNICODE, ELEMENT_TYPE_SIMPLE},
-		
-	{ E_CONTACT_BIRTH_DATE,  PT_SYSTIME, PR_BIRTHDAY, ELEMENT_TYPE_COMPLEX},
-	{ E_CONTACT_ANNIVERSARY, PT_SYSTIME, PR_WEDDING_ANNIVERSARY, ELEMENT_TYPE_COMPLEX},
-		  
-	{ E_CONTACT_NOTE, PT_UNICODE, PR_BODY_UNICODE, ELEMENT_TYPE_SIMPLE},
-
-	{ E_CONTACT_ADDRESS_HOME, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x801a), ELEMENT_TYPE_COMPLEX},
-	{ E_CONTACT_ADDRESS_WORK, PT_UNICODE, PROP_TAG(PT_UNICODE, 0x801c), ELEMENT_TYPE_COMPLEX},
-//		{ E_CONTACT_BOOK_URI, ELEMENT_TYPE_SIMPLE, "book_uri"}
-//		{ E_CONTACT_CATEGORIES, },		
-	};
-
-static int maplen = G_N_ELEMENTS(mappings);
 
 struct _EBookBackendMAPIGALPrivate
 {
-	char *profile;
+	gchar *profile;
+	ExchangeMapiConnection *conn;
+
 	mapi_id_t fid;
-	int mode;
+	gint mode;
 	gboolean marked_for_offline;
 	GThread *build_cache_thread;
 	gboolean kill_cache_build;
 	gboolean is_cache_ready;
 	gboolean is_summary_ready;
 	gboolean is_writable;
-	char *uri;
-	char *book_name;
-	
+	gchar *uri;
+	gchar *book_name;
+
 	GMutex *lock;
-	char *summary_file_name;
+	gchar *summary_file_name;
 	EBookBackendSummary *summary;
 	EBookBackendCache *cache;
-
 };
 
 #define SUMMARY_FLUSH_TIMEOUT 5000
 
-static char *
+static gchar *
 e_book_backend_mapi_gal_get_static_capabilities (EBookBackend *backend)
 {
-	if(enable_debug)
+	if (enable_debug)
 		printf("mapi get_static_capabilities\n");
 	//FIXME: Implement this.
-	
+
 	return g_strdup ("net,bulk-removes,do-initial-query,contact-lists");
 }
 
@@ -165,16 +103,91 @@ book_view_notify_status (EDataBookView *view, const gchar *status)
 	e_data_book_view_notify_status_message (view, status);
 }
 
+static guint32
+current_time_ms (void)
+{
+	GTimeVal tv;
+
+	g_get_current_time (&tv);
+
+	return tv.tv_sec * 1000 + tv.tv_usec / 1000;
+}
+
+struct fetch_gal_data
+{
+	EBookBackendMAPIGAL *ebmapi;
+	EDataBookView *book_view;
+	mapi_id_t fid; /* folder ID of contacts, for named IDs */
+	guint32 last_update; /* when in micro-seconds was done last notification about progress */
+};
+
+static gboolean
+fetch_gal_cb (ExchangeMapiConnection *conn, uint32_t row_index, uint32_t n_rows, struct SRow *aRow, gpointer data)
+{
+	EBookBackendMAPIGALPrivate *priv;
+	struct fetch_gal_data *fgd = data;
+	EContact *contact;
+	gchar *uid;
+	guint32 current_time;
+
+	g_return_val_if_fail (conn != NULL, FALSE);
+	g_return_val_if_fail (aRow != NULL, FALSE);
+	g_return_val_if_fail (data != NULL, FALSE);
+
+	priv = fgd->ebmapi->priv;
+	if (priv->kill_cache_build)
+		return FALSE;
+
+	contact = mapi_book_utils_contact_from_props (conn, fgd->fid, NULL, aRow);
+	if (!contact) {
+		/* just ignore them */
+		return TRUE;
+	}
+
+	uid = g_strdup_printf ("%d", row_index);
+	e_contact_set (contact, E_CONTACT_UID, uid);
+	g_free (uid);
+
+	e_book_backend_cache_add_contact (priv->cache, contact);
+	e_book_backend_summary_add_contact (priv->summary, contact);
+
+	if (!fgd->book_view)
+		fgd->book_view = find_book_view (fgd->ebmapi);
+
+	if (fgd->book_view)
+		e_data_book_view_notify_update (fgd->book_view, contact);
+
+	current_time = current_time_ms ();
+	if (fgd->book_view && current_time - fgd->last_update >= 333) {
+		gchar *status_msg;
+
+		if (n_rows > 0) {
+			/* To translators : This is used to cache the downloaded contacts from GAL.
+			   The first %d is an index of the GAL entry,
+			   the second %d is total count of entries in GAL. */
+			status_msg = g_strdup_printf (_("Caching GAL entry %d/%d"), row_index, n_rows);
+		} else {
+			/* To translators : This is used to cache the downloaded contacts from GAL.
+			   %d is an index of the GAL entry. */
+			status_msg = g_strdup_printf (_("Caching GAL entry %d"), row_index);
+		}
+		book_view_notify_status (fgd->book_view, status_msg);
+		g_free (status_msg);
+		fgd->last_update = current_time;
+	}
+
+	g_object_unref (contact);
+
+	return TRUE;
+}
+
 static gpointer
 build_cache (EBookBackendMAPIGAL *ebmapi)
 {
-	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) ebmapi)->priv;
-	char *tmp;
-	GPtrArray *contacts_array = g_ptr_array_new();
-	gint i = 0;
-	gchar *status_msg;
-	EDataBookView *book_view;
-	
+	EBookBackendMAPIGALPrivate *priv = ebmapi->priv;
+	gchar *tmp;
+	struct fetch_gal_data fgd = { 0 };
+
 	//FIXME: What if book view is NULL? Can it be? Check that.
 	if (!priv->cache) {
 		printf("Caching for the first time\n");
@@ -182,68 +195,38 @@ build_cache (EBookBackendMAPIGAL *ebmapi)
 	}
 
 	if (!priv->summary) {
-		priv->summary = e_book_backend_summary_new (priv->summary_file_name, 
+		priv->summary = e_book_backend_summary_new (priv->summary_file_name,
 							    SUMMARY_FLUSH_TIMEOUT);
 		printf("Summary file name is %s\n", priv->summary_file_name);
 	}
 
-	exchange_mapi_util_get_gal (contacts_array);
-	
+	fgd.ebmapi = ebmapi;
+	fgd.book_view = find_book_view (ebmapi);
+	fgd.fid = exchange_mapi_connection_get_default_folder_id (priv->conn, olFolderContacts);
+	fgd.last_update = current_time_ms ();
+
 	e_file_cache_freeze_changes (E_FILE_CACHE (priv->cache));
+	exchange_mapi_connection_fetch_gal (priv->conn,
+					mapi_book_utils_get_prop_list, GET_ALL_KNOWN_IDS,
+					fetch_gal_cb, &fgd);
 
-	book_view = find_book_view (ebmapi);
-
-	if (book_view)
-		e_data_book_view_notify_complete (book_view, 
-						  GNOME_Evolution_Addressbook_Success);
-
-	for (i = 0; i < contacts_array->len && !priv->kill_cache_build; i++) {
-		EContact *contact = e_contact_new ();
-		ExchangeMAPIGALEntry *gal_entry = contacts_array->pdata[i];
-		char *uid;
-
-		uid = g_strdup_printf ("%d", i);
-
-		e_contact_set (contact, E_CONTACT_UID, uid);
-		e_contact_set (contact, E_CONTACT_FULL_NAME, gal_entry->name);
-		e_contact_set (contact, E_CONTACT_EMAIL_1, gal_entry->email);
-
-		e_book_backend_cache_add_contact (priv->cache, contact);
-		e_book_backend_summary_add_contact (priv->summary, contact);
-
-		if (book_view && (i % 200 == 0)) {
-			/* To translators : This is used to cache the downloaded contacts from GAL.
-			   First %d : Number of contacts cached till now.
-			   Second %d : Total number of contacts which need to be cached.
-			   So (%d/%d) displays the progress.
-			   Example: Caching the GAL entries (1200/50000)...
-			*/
-			status_msg = g_strdup_printf (_("Caching the GAL entries (%d/%d)... "),
-							 i, contacts_array->len);
-			book_view_notify_status (book_view, status_msg);
-			g_free (status_msg);
-		}
-
-		g_object_unref(contact);
-		g_free (uid);
+	if (fgd.book_view) {
+		e_data_book_view_notify_complete (fgd.book_view, GNOME_Evolution_Addressbook_Success);
+		e_data_book_view_unref (fgd.book_view);
 	}
 
-	if (book_view) {
-		e_data_book_view_notify_complete (book_view,
-						  GNOME_Evolution_Addressbook_Success);
-		e_data_book_view_unref (book_view);
-	}
-	
-	g_ptr_array_free (contacts_array, TRUE);
-
-	tmp = g_strdup_printf("%d", priv->kill_cache_build ? 0 : (int)time (NULL));
+	tmp = g_strdup_printf("%d", priv->kill_cache_build ? 0 : (gint)time (NULL));
 	e_book_backend_cache_set_time (priv->cache, tmp);
 	printf("setting time  %s\n", tmp);
 	g_free (tmp);
+
 	e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
+
 	e_book_backend_summary_save (priv->summary);
+
 	priv->is_cache_ready = !priv->kill_cache_build;
 	priv->is_summary_ready = !priv->kill_cache_build;
+
 	return NULL;
 }
 
@@ -253,10 +236,14 @@ e_book_backend_mapi_gal_get_supported_fields (EBookBackend *backend,
 		      guint32	    opid)
 
 {
+	GList *fields;
+
+	fields = mapi_book_utils_get_supported_fields ();
 	e_data_book_respond_get_supported_fields (book,
 						  opid,
 						  GNOME_Evolution_Addressbook_Success,
-						  supported_fields);
+						  fields);
+	g_list_free (fields);
 }
 
 static void
@@ -278,34 +265,39 @@ static void
 e_book_backend_mapi_gal_authenticate_user (EBookBackend *backend,
 					    EDataBook    *book,
 					    guint32       opid,
-					    const char *user,
-					    const char *passwd,
-					    const char *auth_method)
+					    const gchar *user,
+					    const gchar *passwd,
+					    const gchar *auth_method)
 {
 	EBookBackendMAPIGAL *ebmapi = (EBookBackendMAPIGAL *) backend;
 	EBookBackendMAPIGALPrivate *priv = ebmapi->priv;
-	
+
 	if (enable_debug) {
 		printf ("mapi: authenticate user\n");
-	}	
+	}
 
-	
 	switch (priv->mode) {
 	case GNOME_Evolution_Addressbook_MODE_LOCAL:
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_set_is_writable (E_BOOK_BACKEND(backend), FALSE);
-		e_book_backend_notify_connection_status (backend, FALSE); 
-		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success); 
+		e_book_backend_notify_connection_status (backend, FALSE);
+		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success);
 		return;
-		
+
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
-		
-		if (!exchange_mapi_connection_new (priv->profile, passwd))
+
+		priv->conn = exchange_mapi_connection_new (priv->profile, passwd);
+		if (!priv->conn) {
+			priv->conn = exchange_mapi_connection_find (priv->profile);
+			if (priv->conn && !exchange_mapi_connection_connected (priv->conn))
+				exchange_mapi_connection_reconnect (priv->conn, passwd);
+		}
+		if (!priv->conn)
 			return e_data_book_respond_authenticate_user (book, opid,GNOME_Evolution_Addressbook_OtherError);
 
 		if (priv->cache && priv->is_cache_ready) {
 			printf("FIXME: Should check for an update in the cache\n");
-//			g_thread_create ((GThreadFunc) update_cache, 
+//			g_thread_create ((GThreadFunc) update_cache,
 	//					  backend, FALSE, backend);
 		} else if (priv->marked_for_offline && !priv->is_cache_ready) {
 			if (!priv->build_cache_thread) {
@@ -314,21 +306,21 @@ e_book_backend_mapi_gal_authenticate_user (EBookBackend *backend,
 				priv->kill_cache_build = FALSE;
 				priv->build_cache_thread = g_thread_create ((GThreadFunc) build_cache, ebmapi, TRUE, NULL);
 			}
-		} 
+		}
 		e_book_backend_set_is_writable (backend, FALSE);
 		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success);
 		return;
-		
+
 	default :
 		break;
-	}	
+	}
 }
 
-static char *
-get_filename_from_uri (const char *uri, const char *file)
+static gchar *
+get_filename_from_uri (const gchar *uri, const gchar *file)
 {
-	char *mangled_uri, *filename;
-	int i;
+	gchar *mangled_uri, *filename;
+	gint i;
 
 	/* mangle the URI to not contain invalid characters */
 	mangled_uri = g_strdup (uri);
@@ -390,40 +382,43 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 {
 	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) backend)->priv;
 	const gchar *offline, *tmp;
-	char **tokens;
-	char *uri = NULL;
+	gchar **tokens;
+	gchar *uri = NULL;
+
 	if (enable_debug)
 		printf("MAPI load source\n");
+
+	if (e_book_backend_is_loaded (backend))
+		return GNOME_Evolution_Addressbook_Success;
+
 	offline = e_source_get_property (source, "offline_sync");
 	if (offline  && g_str_equal (offline, "1"))
 		priv->marked_for_offline = TRUE;
 
-
-
 	/* Either we are in Online mode or this is marked for offline */
-	
+
 	priv->uri = g_strdup (e_source_get_uri (source));
 
 	tokens = g_strsplit (priv->uri, ";", 2);
-  	if (tokens[0])
- 		uri = g_strdup (tokens [0]);
-  	priv->book_name  = g_strdup (tokens[1]);
-  	if (priv->book_name == NULL) {
+	if (tokens[0])
+		uri = g_strdup (tokens [0]);
+	priv->book_name  = g_strdup (tokens[1]);
+	if (priv->book_name == NULL) {
 		g_warning ("Bookname is null for %s\n", uri);
-  		return GNOME_Evolution_Addressbook_OtherError;
+		return GNOME_Evolution_Addressbook_OtherError;
 	}
-  	g_strfreev (tokens);
+	g_strfreev (tokens);
 
 	if (priv->mode ==  GNOME_Evolution_Addressbook_MODE_LOCAL &&
 	    !priv->marked_for_offline ) {
 		return GNOME_Evolution_Addressbook_OfflineUnavailable;
 	}
-	
+
 	if (priv->marked_for_offline) {
- 		priv->summary_file_name = get_filename_from_uri (priv->uri, "cache.summary"); 
+		priv->summary_file_name = get_filename_from_uri (priv->uri, "cache.summary");
 		if (g_file_test (priv->summary_file_name, G_FILE_TEST_EXISTS)) {
 			printf("Loading the summary\n");
-			priv->summary = e_book_backend_summary_new (priv->summary_file_name, 
+			priv->summary = e_book_backend_summary_new (priv->summary_file_name,
 								    SUMMARY_FLUSH_TIMEOUT);
 			e_book_backend_summary_load (priv->summary);
 			priv->is_summary_ready = TRUE;
@@ -447,7 +442,7 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 
 	g_free (uri);
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (backend), TRUE);
-	e_book_backend_set_is_writable (backend, FALSE);	
+	e_book_backend_set_is_writable (backend, FALSE);
 	if (priv->mode ==  GNOME_Evolution_Addressbook_MODE_LOCAL) {
 		e_book_backend_set_is_writable (backend, FALSE);
 		e_book_backend_notify_writable (backend, FALSE);
@@ -459,7 +454,7 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 	} else {
 		e_book_backend_notify_connection_status (backend, TRUE);
 	}
-	
+
 	priv->profile = g_strdup (e_source_get_property (source, "profile"));
 	exchange_mapi_util_mapi_id_from_string (e_source_get_property (source, "folder-id"), &priv->fid);
 
@@ -474,21 +469,20 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (backend), TRUE);
 	e_book_backend_notify_connection_status (E_BOOK_BACKEND (backend), TRUE);
 
-
 	if (enable_debug)
 		printf("For profile %s and folder %s - %016" G_GINT64_MODIFIER "X\n", priv->profile, tmp, priv->fid);
 
 	return GNOME_Evolution_Addressbook_Success;
 }
 
-static void 
+static void
 e_book_backend_mapi_gal_set_mode (EBookBackend *backend, GNOME_Evolution_Addressbook_BookMode mode)
 {
 	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) backend)->priv;
 
-	if(enable_debug)
+	if (enable_debug)
 		printf("mapi: set_mode \n");
-	
+
 	priv->mode = mode;
 	if (e_book_backend_is_loaded (backend)) {
 		if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
@@ -503,7 +497,7 @@ e_book_backend_mapi_gal_set_mode (EBookBackend *backend, GNOME_Evolution_Address
 			e_book_backend_notify_connection_status (backend, TRUE);
 //			e_book_backend_notify_auth_required (backend); //FIXME: WTH is this required.
 		}
-	}	
+	}
 }
 
 static void
@@ -511,11 +505,15 @@ e_book_backend_mapi_gal_dispose (GObject *object)
 {
 	/* FIXME : provide implmentation */
 	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) object)->priv;
-	
+
 	if (priv->profile) {
 		g_free (priv->profile);
 		priv->profile = NULL;
 	}
+	if (priv->conn) {
+		g_object_unref (priv->conn);
+		priv->conn = NULL;
+	}
 	if (priv->uri) {
 		g_free (priv->uri);
 		priv->uri = NULL;
@@ -540,97 +538,23 @@ get_closure (EDataBookView *book_view)
 	return g_object_get_data (G_OBJECT (book_view), "closure");
 }
 
-#if 0
-static EContact *
-emapidump_gal (struct SRow *gal_entry)
-{
-	EContact *contact = e_contact_new ();
-	int i;
-	
-//	exchange_mapi_debug_property_dump (properties);
-	for (i=1; i<maplen; i++) {
-		gpointer value;
-
-		/* can cast it, no writing to the value; and it'll be freed not before the end of this function */
-		value = (gpointer) exchange_mapi_util_find_row_propval (gal_entry, mappings[i].mapi_id);
-		if (mappings[i].element_type == PT_UNICODE && mappings[i].contact_type == ELEMENT_TYPE_SIMPLE) {
-			if (value)
-				e_contact_set (contact, mappings[i].field_id, value);
-		} else if (mappings[i].contact_type == ELEMENT_TYPE_SIMPLE) {
-			if (value && mappings[i].element_type == PT_SYSTIME) {
-				struct FILETIME *t = value;
-				time_t time;
-				NTTIME nt;
-				char buff[129];
-
-				nt = t->dwHighDateTime;
-				nt = nt << 32;
-				nt |= t->dwLowDateTime;
-				time = nt_time_to_unix (nt);
-				e_contact_set (contact, mappings[i].field_id, ctime_r (&time, buff));
-			} else {
-				printf("Nothing is printed\n");
-			}
-		}
-	}
-}
-
-static gboolean
-create_gal_contact_cb (FetchItemsCallbackData *item_data, gpointer data)
-{
-	EDataBookView *book_view = data;
-	BESearchClosure *closure = get_closure (book_view);
-	EBookBackendMAPIGAL *be = closure->bg;
-	EContact *contact;
-	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) be)->priv;
-	char *suid;
-	GSList *l;
-	int counter;
-	
-	if (!e_flag_is_set (closure->running)) {
-		printf("Might be that the operation is cancelled. Lets ask our parent also to do.\n");
-		return FALSE;
-	}
-	
-//	contact = emapidump_contact (item_data->properties);
-	for (l=item_data->gallist; l; l=l->next) {
-		struct SRow *SRow = (struct SRow *) (l->data);
-		counter++;
-		contact = emapidump_gal (SRow);
-		suid = exchange_mapi_util_mapi_ids_to_uid (item_data->fid, item_data->mid);
-	
-		if (contact) {
-			/* UID of the contact is nothing but the concatenated string of hex id of folder and the message.*/
-			e_contact_set (contact, E_CONTACT_UID, suid);		
-			e_contact_set (contact, E_CONTACT_BOOK_URI, priv->uri);
-			e_data_book_view_notify_update (book_view, contact);
-			g_object_unref(contact);
-		}
-
-		g_free (suid);
-	}
-	g_print ("\n The counter for the above data is %d\n", counter);
-	return TRUE;
-}
-#endif
-
 static void
-get_contacts_from_cache (EBookBackendMAPIGAL *ebmapi, 
-			 const char *query,
+get_contacts_from_cache (EBookBackendMAPIGAL *ebmapi,
+			 const gchar *query,
 			 GPtrArray *ids,
-			 EDataBookView *book_view, 
+			 EDataBookView *book_view,
 			 BESearchClosure *closure)
 {
-	int i;
+	gint i;
 
 	for (i = 0; i < ids->len; i ++) {
-		char *uid;
-		EContact *contact; 
+		gchar *uid;
+		EContact *contact;
 
                 if (!e_flag_is_set (closure->running))
                         break;
 
- 		uid = g_ptr_array_index (ids, i);
+		uid = g_ptr_array_index (ids, i);
 		contact = e_book_backend_cache_get_contact (ebmapi->priv->cache, uid);
 		if (contact) {
 			e_data_book_view_notify_update (book_view, contact);
@@ -638,19 +562,19 @@ get_contacts_from_cache (EBookBackendMAPIGAL *ebmapi,
 		}
 	}
 	if (e_flag_is_set (closure->running))
-		e_data_book_view_notify_complete (book_view, 
+		e_data_book_view_notify_complete (book_view,
 						  GNOME_Evolution_Addressbook_Success);
 }
 #if 0
 static gboolean
-build_restriction_emails_contains (struct mapi_SRestriction *res, 
-				   const char *query)
+build_restriction_emails_contains (struct mapi_SRestriction *res,
+				   const gchar *query)
 {
-	char *email=NULL, *tmp, *tmp1;
+	gchar *email=NULL, *tmp, *tmp1;
 
 	/* This currently supports "email foo@bar.soo" */
 	tmp = strdup (query);
-	
+
 	tmp = strstr (tmp, "email");
 	if (tmp ) {
 		tmp = strchr (tmp, '\"');
@@ -666,7 +590,6 @@ build_restriction_emails_contains (struct mapi_SRestriction *res,
 			}
 		}
 	}
-	
 
 	if (email==NULL || !strchr (email, '@'))
 		return FALSE;
@@ -682,17 +605,17 @@ build_restriction_emails_contains (struct mapi_SRestriction *res,
 #endif
 
 static gboolean
-build_multiple_restriction_emails_contains (struct mapi_SRestriction *res, 
-				            struct mapi_SRestriction_or *or_res, 
-					    const char *query)
+build_multiple_restriction_emails_contains (struct mapi_SRestriction *res,
+					    struct mapi_SRestriction_or *or_res,
+					    const gchar *query)
 {
-	char *email=NULL, *tmp, *tmp1;
+	gchar *email=NULL, *tmp, *tmp1;
 	//Number of restriction to apply
-	unsigned int res_count = 6;
+	guint res_count = 6;
 
 	/* This currently supports "email foo@bar.soo" */
 	tmp = strdup (query);
-	
+
 	tmp = strstr (tmp, "email");
 	if (tmp ) {
 		tmp = strchr (tmp, '\"');
@@ -760,42 +683,42 @@ book_view_thread (gpointer data)
 	BESearchClosure *closure = get_closure (book_view);
 	EBookBackendMAPIGAL *backend = closure->bg;
 	EBookBackendMAPIGALPrivate *priv = backend->priv;
-	const char *query = NULL;
+	const gchar *query = NULL;
 	GPtrArray *ids = NULL;
 	GList *contacts = NULL, *temp_list = NULL;
 	//Number of multiple restriction to apply
-	unsigned int res_count = 6;
-	
+	guint res_count = 6;
+
 	if (enable_debug)
 		printf("mapi: book view\n");
-	
+
 	g_object_ref (book_view);
 	e_flag_set (closure->running);
-						
-	book_view_notify_status (book_view, "Searching...");
+
+	book_view_notify_status (book_view, _("Searching"));
 	query = e_data_book_view_get_card_query (book_view);
 
 	if (!find_book_view (backend))
 		e_book_backend_add_book_view (E_BOOK_BACKEND (backend), book_view);
-						
+
 	switch (priv->mode) {
 		case GNOME_Evolution_Addressbook_MODE_REMOTE:
-			if (!exchange_mapi_connection_exists ()) {
+			if (!priv->conn) {
 				e_book_backend_notify_auth_required (E_BOOK_BACKEND (backend));
 				e_data_book_view_notify_complete (book_view,
 							GNOME_Evolution_Addressbook_AuthenticationRequired);
 				g_object_unref (book_view);
 				return;
 			}
-		
+
 			if (priv->marked_for_offline && !priv->is_cache_ready) {
 				/* To translators : Here Evolution MAPI downloads the entries from the GAL server */
-				book_view_notify_status (book_view, _("Downloading GAL entries from server..."));
+				book_view_notify_status (book_view, _("Downloading GAL entries from server…"));
 				return;
 			}
 
 			if (priv->marked_for_offline && priv->cache && priv->is_cache_ready) {
-				if (priv->is_summary_ready && 
+				if (priv->is_summary_ready &&
 				    e_book_backend_summary_is_summary_query (priv->summary, query)) {
 					if (enable_debug)
 						printf ("reading the contacts from summary \n");
@@ -807,11 +730,11 @@ book_view_thread (gpointer data)
 					g_object_unref (book_view);
 					break;
 				}
-			
+
 				printf("Summary seems to be not there or not a summary query, lets fetch from cache directly\n");
-			
+
 				/* We are already cached. Lets return from there. */
-				contacts = e_book_backend_cache_get_contacts (priv->cache, 
+				contacts = e_book_backend_cache_get_contacts (priv->cache,
 								      query);
 				temp_list = contacts;
 				for (; contacts != NULL; contacts = g_list_next(contacts)) {
@@ -819,28 +742,28 @@ book_view_thread (gpointer data)
 						for (;contacts != NULL; contacts = g_list_next (contacts))
 							g_object_unref (contacts->data);
 						break;
-					}							
-					e_data_book_view_notify_update (book_view, 
+					}
+					e_data_book_view_notify_update (book_view,
 									E_CONTACT(contacts->data));
 					g_object_unref (contacts->data);
 				}
 				if (e_flag_is_set (closure->running))
-					e_data_book_view_notify_complete (book_view, 
+					e_data_book_view_notify_complete (book_view,
 									  GNOME_Evolution_Addressbook_Success);
 				if (temp_list)
 					 g_list_free (temp_list);
 				g_object_unref (book_view);
 				break;
 			}
-		
+
 			if (e_book_backend_summary_is_summary_query (priv->summary, query)) {
 				or_res = g_new (struct mapi_SRestriction_or, res_count);
-		
+
 				if (!build_multiple_restriction_emails_contains (&res, or_res, query)) {
-					e_data_book_view_notify_complete (book_view, 
+					e_data_book_view_notify_complete (book_view,
 								  GNOME_Evolution_Addressbook_OtherError);
-					return ;
-				} 
+					return;
+				}
 
 			if (e_flag_is_set (closure->running))
 				e_data_book_view_notify_complete (book_view,
@@ -849,9 +772,9 @@ book_view_thread (gpointer data)
 			break;
 		}
 	}
-	
+
 	if (book_view)
-		e_data_book_view_notify_complete (book_view, 
+		e_data_book_view_notify_complete (book_view,
 						  GNOME_Evolution_Addressbook_Success);
 	return;
 }
@@ -882,7 +805,7 @@ static void
 e_book_backend_mapi_gal_get_contact (EBookBackend *backend,
 				     EDataBook    *book,
 				     guint32       opid,
-				     const char   *id)
+				     const gchar   *id)
 {
 	if (enable_debug)
 		printf ("mapi: get contact %s\n", id);
@@ -894,11 +817,11 @@ static void
 e_book_backend_mapi_gal_get_contact_list (EBookBackend *backend,
 					    EDataBook    *book,
 					    guint32       opid,
-					    const char   *query )
+					    const gchar   *query )
 {
 	/*EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) backend)->priv;*/
 
-	if(enable_debug)
+	if (enable_debug)
 		printf("mapi: get contact list %s\n", query);
 
 	e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_RepositoryOffline,
@@ -917,21 +840,21 @@ e_book_backend_mapi_gal_start_book_view (EBookBackend  *backend,
 		printf ("mapi: start_book_view...\n");
 	closure->thread = g_thread_create ((GThreadFunc) book_view_thread, book_view, FALSE, NULL);
 	e_flag_wait (closure->running);
-	
-	/* at this point we know the book view thread is actually running */	
+
+	/* at this point we know the book view thread is actually running */
 }
 
 static void
 e_book_backend_mapi_gal_stop_book_view (EBookBackend  *backend,
 					  EDataBookView *book_view)
 {
-	if(enable_debug)
-		printf("mapi: stop book view\n");	
+	if (enable_debug)
+		printf("mapi: stop book view\n");
 	/* FIXME : provide implmentation */
 }
 
 static void
-e_book_backend_mapi_gal_get_changes (EBookBackend *backend, EDataBook *book, guint32 opid, const char *change_id)
+e_book_backend_mapi_gal_get_changes (EBookBackend *backend, EDataBook *book, guint32 opid, const gchar *change_id)
 {
 	if (enable_debug)
 		printf ("mapi: get changes\n");
@@ -939,12 +862,12 @@ e_book_backend_mapi_gal_get_changes (EBookBackend *backend, EDataBook *book, gui
 	e_data_book_respond_get_changes (book, opid, GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
 }
 
-static void 
+static void
 e_book_backend_mapi_gal_get_supported_auth_methods (EBookBackend *backend, EDataBook *book, guint32 opid)
 {
 	GList *auth_methods = NULL;
-	char *auth_method;
-	
+	gchar *auth_method;
+
 	if (enable_debug)
 		printf ("mapi get_supported_auth_methods...\n");
 
@@ -953,9 +876,9 @@ e_book_backend_mapi_gal_get_supported_auth_methods (EBookBackend *backend, EData
 	e_data_book_respond_get_supported_auth_methods (book,
 							opid,
 							GNOME_Evolution_Addressbook_Success,
-							auth_methods);  
+							auth_methods);
 	g_free (auth_method);
-	g_list_free (auth_methods);	
+	g_list_free (auth_methods);
 }
 
 static GNOME_Evolution_Addressbook_CallStatus
@@ -976,10 +899,9 @@ static void e_book_backend_mapi_gal_class_init (EBookBackendMAPIGALClass *klass)
 {
 	GObjectClass  *object_class = G_OBJECT_CLASS (klass);
 	EBookBackendClass *parent_class;
-	gint i;
-	
+
 	e_book_backend_mapi_gal_parent_class = g_type_class_peek_parent (klass);
-	
+
 	parent_class = E_BOOK_BACKEND_CLASS (klass);
 
 	/* Set the virtual methods. */
@@ -1002,14 +924,6 @@ static void e_book_backend_mapi_gal_class_init (EBookBackendMAPIGALClass *klass)
 	parent_class->set_mode                   = e_book_backend_mapi_gal_set_mode;
 	parent_class->remove			 = e_book_backend_mapi_gal_remove;
 	object_class->dispose                    = e_book_backend_mapi_gal_dispose;
-
-
-	supported_fields = NULL;
-	for (i = 0; i < maplen; i++) {
-		supported_fields = g_list_append (supported_fields,
-						  (gchar *)e_contact_field_name (mappings[i].field_id));
-	}
-	supported_fields = g_list_append (supported_fields, (gpointer) "file_as");
 }
 
 /**
@@ -1031,7 +945,7 @@ e_book_backend_mapi_gal_new (void)
 static void	e_book_backend_mapi_gal_init (EBookBackendMAPIGAL *backend)
 {
 	EBookBackendMAPIGALPrivate *priv;
-  
+
 	priv= g_new0 (EBookBackendMAPIGALPrivate, 1);
 	/* Priv Struct init */
 	backend->priv = priv;
@@ -1043,9 +957,9 @@ static void	e_book_backend_mapi_gal_init (EBookBackendMAPIGAL *backend)
 	priv->cache = NULL;
 	priv->is_summary_ready = FALSE;
 	priv->is_cache_ready = FALSE;
-	
+
 */	if (g_getenv ("MAPI_DEBUG"))
 		enable_debug = TRUE;
 	else
-		enable_debug = FALSE;	
+		enable_debug = FALSE;
 }
